/* SWI Prolog system-dependent predicates for CLAIR.
   Copyright (C) 2002-2005 Roberto Bagnara <bagnara@cs.unipr.it>

This file is part of the Combined Language and Abstract Interpretation
Resource (CLAIR).

CLAIR is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.

CLAIR is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

For the most up-to-date information see the CLAIR site:
http://www.cs.unipr.it/clair/ . */

/*
**    CLAIR: `sysdep' module
**
**    This module contains the system-dependent predicates of CLAIR.
**    All the other modules are written in ISO Prolog, so that this
**    is the only module to modify to port CLAIR to a standard Prolog
**    engine.
**
**    Predicates defined in this module:
**
**    buildArgsList/2, callSystem/1, cpuTime/1, existingFile/1,
**    invokeBrowser/1, invokeEditor/1, shellCommand/1, showDirectory/1.
*/


/*
**    cpuTime(X)
**        - X is unified with the CPU time, in milliseconds, employed
**            by the process since the last call to cpuTime/1 or, for
**            the first call to cpuTime/1, since the process creation.
*/

cpuTime(X) :-
    statistics(runtime, [_, X]).

/*
**    callSystem(X)
**       - X is a list of ASCII codes.
**
**    Executes (passing it to the shell) the command corresponding
**    to the string represented by X.
*/

callSystem(X) :- name(Y, X), unix(system(Y)).

/*
**    shellCommand(Args)
**       - Args is a (possibly empty) list of atoms representing
**         (parts of) shell commands.
**
**    If Args is the empty list, an interactive shell is invoked.
**    Otherwise, the shell command represented by Args is executed.
*/

shellCommand([]) :- !,
    callSystem("csh").

shellCommand(Args) :-
    buildArgsList(Args, L),
    callSystem(L).

buildArgsList([A], L) :- !, name(A, L).
buildArgsList([A|Args], L) :-
    name(A, L1), buildArgsList(Args, L2),
    append(L1, [32|L2], L).
buildArgsList([], []).

/*
**    showDirectory(Args)
**       - Args is a (possibly empty) list of atoms representing
**         options for the "show directory" command.
*/

showDirectory(Args) :-
    shellCommand([ls|Args]).

/*
**    invokeEditor(Args)
**       - Args is a (possibly empty) list of atoms representing
**         options for the editor.
*/

invokeEditor(Args) :-
    shellCommand(['emacs'|Args]).

/*
**    invokeBrowser(Args)
**       - Args is a (possibly empty) list of atoms representing
**         options for the browser.
*/

invokeBrowser(Args) :-
    shellCommand([less|Args]).
